@@ -5,13 +5,26 @@ CURDIR=`pwd`
 
 echo "Checking libmaxminddb ..."
 
+DLCMD=
+source dist/functions.sh 2>/dev/null
+if [ $? != 0 ] ; then
+    . dist/functions.sh
+    if [ $? != 0 ] ; then
+        echo [ERROR] Can not include 'functions.sh'.
+        exit 1
+    fi
+fi
+
+
 VERSION=1.3.2
 URL=https://github.com/maxmind/libmaxminddb/archive/$VERSION.tar.gz
 
 if [ ! -f "libmaxminddb/src/.libs/libmaxminddb.a" ] ; then
+    
+    detectdlcmd
     echo -e "\033[38;5;148mDownloading libmaxminddb version $VERSION and building, it will take several minutes ...\033[39m"
 
-    wget  --no-check-certificate  -O mmdb.tgz   $URL
+    $DLCMD mmdb.tgz   $URL
     tar xf mmdb.tgz
     mv libmaxminddb-$VERSION ../libmaxminddb
     cd ../libmaxminddb
