@@ -40,7 +40,8 @@
     T(const T&);               \
     void operator=(const T&);
 
-#define LS_ZERO_FILL(x, y)     memset(&x, 0, (char *)(&y+1) - (char *)&x)
+#define LS_ZERO_FILL(x, y)  memset((char *)this + ((char *)&x - (char *)this), \
+                                                0, (char *)(&y+1) - (char *)&x)
 
 #define ls_inline           static inline
 #define ls_always_inline    static inline __attribute__((always_inline))
