@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2018  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -288,16 +288,21 @@ int ExtConn::onError()
 }
 
 
+void ExtConn::keepAliveOff()
+{
+    if (!isToClose())
+    {
+        setToClose(1);
+        m_pWorker->incLingerConn();
+    }
+}
+
+
 int ExtConn::markToClose()
 {
     if (getReq())
     {
-        if (!isToClose())
-        {
-            setToClose(1);
-            m_pWorker->incLingerConn();
-            return 0;
-        }
+        keepAliveOff();
     }
     else
     {
