@@ -109,6 +109,12 @@ void HttpLog::init()
 }
 
 
+void HttpLog::updateLogPatternWithPid(int pid)
+{
+    lsnprintf( s_logPattern, sizeof(s_logPattern), "%%d [%%p] [%d] %%m", pid);
+}
+
+
 bool HttpLog::isEnabled(Logger *pLogger, int level)
 {
     if (pLogger)
@@ -150,6 +156,12 @@ void HttpLog::toggleDebugLog()
 }
 
 
+int  HttpLog::getLogLevel()
+{
+    return logger()->getLevel();
+}
+
+
 void HttpLog::setLogLevel(int level)
 {
     logger()->setLevel(level);
@@ -180,8 +192,7 @@ LOG4CXX_NS::Logger *HttpLog::getErrorLogger()
     return logger();
 }
 
-#define MAX_PATH_LEN                4096
-#define MAX_LOG_LINE_LEN            4096
+#define MAX_PATH_LEN                8192
 int HttpLog::logAccess(const char *pVHost, int len, HttpSession *pSession)
 {
     accessLog()->log(pVHost, len, pSession);
@@ -205,7 +216,7 @@ int HttpLog::checkLogPathValid(const char *org)
     if (len < 5)
         return 0;
 
-    int i;
+    unsigned i;
     for (i=0; i<sizeof(excludeFileList)/ sizeof(char *); ++i)
     {
         int ll = strlen(excludeFileList[i]);
