@@ -300,6 +300,7 @@ private:
     AutoStr2            m_lastStatPath;
     struct stat         m_lastStat;
     int                 m_lastStatRes;
+    AutoStr2            m_headerLeftOver;
 
     static_file_data_t *m_pUrlStaticFileData;
 
@@ -907,6 +908,9 @@ public:
     void appendReqHeader( const char *pName, int iNameLen,
                           const char *pValue, int iValLen);
 
+    void saveHeaderLeftOver();
+    void restoreHeaderLeftOver();
+
     void classifyUrl();
 
     const Recaptcha *getRecaptcha() const;
