@@ -608,6 +608,13 @@ int RewriteRule::parseRuleSubst(const char *&pRuleStr, const char *pEnd,
         return LS_FAIL;
     }
 
+    if (strncmp(argBegin, "–", argEnd - argBegin) == 0)
+    {
+        HttpLog::parse_warn(s_pCurLine,
+                            "Detects unicode '-', repaces with ascii '-'.");
+        m_flag |= RULE_FLAG_NOREWRITE;
+        return 0;
+    }
     if (strncmp(argBegin, "-", argEnd - argBegin) == 0)
     {
         m_flag |= RULE_FLAG_NOREWRITE;
@@ -1154,6 +1161,29 @@ int RewriteRule::parse(char *&pRule, const RewriteMapList *pMaps)
                 pCur = pLineEnd + 1;
                 return ret;
             }
+            else if ((strncasecmp(pCur, "CacheKeyModify", 14) == 0) &&
+                     (isspace(*(pCur + 14))))
+            {
+                // To support CacheKeyModify directive here, we convert into an
+                // equivalent rewrite rule that always matches.
+                //
+                char buf[1024];
+                // Trim leading whitespace:
+                for (pCur = pCur + 15; pCur < pLineEnd && isspace(*pCur); ++pCur)
+                    ;
+                // Trim trailing whitespace:
+                char *pLast;
+                for (pLast = pLineEnd; pLast > pCur && isspace(pLast[-1]); --pLast)
+                    ;
+                const int nw = snprintf(buf, sizeof(buf),
+                    "\"^\" \"-\" [E=\"cache-key-mod:%.*s\"]",
+                    (int) (pLast - pCur), pCur);
+                if (nw < 0 || (size_t) nw >= sizeof(buf))
+                    return LS_FAIL;
+                int ret = parseRule(buf, buf + nw, pMaps);
+                pCur = pLineEnd + 1;
+                return ret;
+            }
             else
             {
                 HttpLog::parse_error(s_pCurLine,  "invalid rewrite directive ");
