@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2018  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -213,7 +213,7 @@ int HttpSession::attachSubSession(HttpSession *pSubSess)
 {
     HioChainStream *pStream = (HioChainStream *)pSubSess->getStream();
     pStream->setParentSession(this);
-    pSubSess->getResp()->setContentLen(LSI_RSP_BODY_SIZE_UNKNOWN);
+    pSubSess->getResp()->setContentLen(LSI_BODY_SIZE_UNKNOWN);
     pStream->setFlag(HIO_FLAG_PASS_THROUGH, 1);
     m_pCurSubSession = pSubSess;
     if (pStream->isWantWrite())
