@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 #include <stddef.h>
 #include <stdint.h>
 #include <string.h>
@@ -20,7 +20,7 @@ const char *const lsquic_enclev2str[] =
 
 
 enum lsquic_version
-lsquic_zero_rtt_version (const unsigned char *buf, size_t bufsz)
+lsquic_sess_resume_version (const unsigned char *buf, size_t bufsz)
 {
     lsquic_ver_tag_t tag;
 
@@ -30,5 +30,5 @@ lsquic_zero_rtt_version (const unsigned char *buf, size_t bufsz)
         return lsquic_tag2ver(tag);
     }
     else
-        return -1;
+        return N_LSQVER;
 }
