@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 #ifndef WIN32
 #include <arpa/inet.h>
 #else
@@ -10,6 +10,8 @@
 #include <string.h>
 #include <sys/queue.h>
 
+#include <openssl/x509.h>
+
 #include "lsquic.h"
 #include "lsquic_types.h"
 #include "lsquic_int_types.h"
@@ -29,6 +31,7 @@
 #include "lsquic_util.h"
 #include "lsquic_hash.h"
 #include "lsquic_conn.h"
+#include "lsxpack_header.h"
 
 #define LSQUIC_LOGGER_MODULE LSQLM_EVENT
 #include "lsquic_logger.h"
@@ -49,9 +52,9 @@ void
 lsquic_ev_log_packet_in (const lsquic_cid_t *cid,
                                         const lsquic_packet_in_t *packet_in)
 {
-    switch (packet_in->pi_flags & (
-                                   PI_FROM_MINI|
-                                                PI_GQUIC))
+    unsigned packet_sz;
+
+    switch (packet_in->pi_flags & (PI_FROM_MINI|PI_GQUIC))
     {
     case PI_FROM_MINI|PI_GQUIC:
         LCID("packet in: %"PRIu64" (from mini)", packet_in->pi_packno);
@@ -62,15 +65,18 @@ lsquic_ev_log_packet_in (const lsquic_cid_t *cid,
             lsquic_packet_in_ecn(packet_in));
         break;
     case PI_GQUIC:
-        LCID("packet in: %"PRIu64", size: %u", packet_in->pi_packno,
-            (unsigned) (packet_in->pi_data_sz + GQUIC_PACKET_HASH_SZ));
+        packet_sz = packet_in->pi_data_sz
+            + (packet_in->pi_flags & PI_DECRYPTED ? GQUIC_PACKET_HASH_SZ : 0);
+        LCID("packet in: %"PRIu64", size: %u", packet_in->pi_packno, packet_sz);
         break;
     default:
+        packet_sz = packet_in->pi_data_sz
+            + (packet_in->pi_flags & PI_DECRYPTED ? IQUIC_TAG_LEN : 0);
         if (packet_in->pi_flags & PI_LOG_QL_BITS)
             LCID("packet in: %"PRIu64", type: %s, size: %u; ecn: %u, spin: %d; "
                 "path: %hhu; Q: %d; L: %d",
                 packet_in->pi_packno, lsquic_hety2str[packet_in->pi_header_type],
-                (unsigned) (packet_in->pi_data_sz + IQUIC_TAG_LEN),
+                packet_sz,
                 lsquic_packet_in_ecn(packet_in),
                 /* spin bit value is only valid for short packet headers */
                 lsquic_packet_in_spin_bit(packet_in), packet_in->pi_path_id,
@@ -80,7 +86,7 @@ lsquic_ev_log_packet_in (const lsquic_cid_t *cid,
             LCID("packet in: %"PRIu64", type: %s, size: %u; ecn: %u, spin: %d; "
                 "path: %hhu",
                 packet_in->pi_packno, lsquic_hety2str[packet_in->pi_header_type],
-                (unsigned) (packet_in->pi_data_sz + IQUIC_TAG_LEN),
+                packet_sz,
                 lsquic_packet_in_ecn(packet_in),
                 /* spin bit value is only valid for short packet headers */
                 lsquic_packet_in_spin_bit(packet_in), packet_in->pi_path_id);
@@ -506,19 +512,19 @@ lsquic_ev_log_generated_http_headers (const lsquic_cid_t *cid,
     }
 
     for (i = 0; i < headers->count; ++i)
-        LCID("  %.*s: %.*s",
-            (int)    headers->headers[i].name.iov_len,
-            (char *) headers->headers[i].name.iov_base,
-            (int)    headers->headers[i].value.iov_len,
-            (char *) headers->headers[i].value.iov_base);
+        if (headers->headers[i].buf)
+            LCID("  %.*s: %.*s",
+                (int)    headers->headers[i].name_len,
+                lsxpack_header_get_name(&headers->headers[i]),
+                (int)    headers->headers[i].val_len,
+                lsxpack_header_get_value(&headers->headers[i]));
 }
 
 
 void
 lsquic_ev_log_generated_http_push_promise (const lsquic_cid_t *cid,
             lsquic_stream_id_t stream_id, lsquic_stream_id_t promised_stream_id, 
-            const struct lsquic_http_headers *headers,
-            const struct lsquic_http_headers *extra_headers)
+            const struct lsquic_http_headers *headers)
 {
     int i;
 
@@ -526,19 +532,12 @@ lsquic_ev_log_generated_http_push_promise (const lsquic_cid_t *cid,
         PRIu64, stream_id, promised_stream_id);
 
     for (i = 0; i < headers->count; ++i)
-        LCID("  %.*s: %.*s",
-            (int)    headers->headers[i].name.iov_len,
-            (char *) headers->headers[i].name.iov_base,
-            (int)    headers->headers[i].value.iov_len,
-            (char *) headers->headers[i].value.iov_base);
-
-    if (extra_headers)
-        for (i = 0; i < extra_headers->count; ++i)
+        if (headers->headers[i].buf)
             LCID("  %.*s: %.*s",
-                (int)    extra_headers->headers[i].name.iov_len,
-                (char *) extra_headers->headers[i].name.iov_base,
-                (int)    extra_headers->headers[i].value.iov_len,
-                (char *) extra_headers->headers[i].value.iov_base);
+                (int)    headers->headers[i].name_len,
+                lsxpack_header_get_name(&headers->headers[i]),
+                (int)    headers->headers[i].val_len,
+                lsxpack_header_get_value(&headers->headers[i]));
 }
 
 void
@@ -558,9 +557,9 @@ lsquic_ev_log_hsk_completed (const lsquic_cid_t *cid)
 
 
 void
-lsquic_ev_log_zero_rtt (const lsquic_cid_t *cid)
+lsquic_ev_log_sess_resume (const lsquic_cid_t *cid)
 {
-    LCID("zero_rtt successful");
+    LCID("sess_resume successful");
 }
 
 
@@ -572,6 +571,24 @@ lsquic_ev_log_check_certs (const lsquic_cid_t *cid, const lsquic_str_t **certs,
 }
 
 
+void
+lsquic_ev_log_cert_chain (const lsquic_cid_t *cid, struct stack_st_X509 *chain)
+{
+    X509_NAME *name;
+    X509 *cert;
+    unsigned i;
+    char buf[0x100];
+
+    for (i = 0; i < sk_X509_num(chain); ++i)
+    {
+        cert = sk_X509_value(chain, i);
+        name = X509_get_subject_name(cert);
+        LCID("cert #%u: name: %s", i,
+                                X509_NAME_oneline(name, buf, sizeof(buf)));
+    }
+}
+
+
 void
 lsquic_ev_log_version_negotiation (const lsquic_cid_t *cid,
                                         const char *action, const char *ver)
