@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 #include <assert.h>
 #include <stddef.h>
 #include <string.h>
@@ -17,7 +17,12 @@ lsquic_qhkdf_expand (const EVP_MD *md, const unsigned char *secret,
 #ifndef NDEBUG
     int s;
 #endif
+    const size_t len = 2 + 1 + 6 + label_len + 1;
+#ifndef WIN32
     unsigned char info[ 2 + 1 + 6 + label_len + 1];
+#else
+    unsigned char info[ 2 + 1 + 6 + UINT8_MAX + 1];
+#endif
 
     info[0] = out_len >> 8;
     info[1] = out_len;
@@ -35,6 +40,6 @@ lsquic_qhkdf_expand (const EVP_MD *md, const unsigned char *secret,
 #else
     (void)
 #endif
-    HKDF_expand(out, out_len, md, secret, secret_len, info, sizeof(info));
+    HKDF_expand(out, out_len, md, secret, secret_len, info, len);
     assert(s);
 }
