@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2020 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_mm.h -- Memory manager.
  *
@@ -33,7 +33,7 @@ struct lsquic_mm {
     struct ack_info     *acki;
     struct {
         struct malo     *stream_frame;  /* For struct stream_frame */
-        struct malo     *stream_rec_arr;/* For struct stream_rec_arr */
+        struct malo     *frame_rec_arr; /* For struct frame_rec_arr */
         struct malo     *mini_conn;     /* For struct mini_conn */
         struct malo     *mini_conn_ietf;/* For struct ietf_mini_conn */
         struct malo     *retry_conn;    /* For struct retry_conn */
