@@ -217,10 +217,126 @@ lsquic_cid_from_packet (const unsigned char *buf, size_t bufsz,
 }
 
 
+int
+lsquic_dcid_from_packet (const unsigned char *buf, size_t bufsz,
+                                unsigned server_cid_len, unsigned *cid_len)
+{
+    const unsigned char *p;
+    unsigned dcil, scil;
+
+    if (bufsz < 9)
+        return -1;
+
+    switch (buf[0] >> 3)
+    {
+    /* Xs vary, Gs are iGnored: */
+    /* 1X11 XGGG: */
+    case (0x80|0x40|0x20|0x10|0x08) >> 3:
+    case (0x80|0x00|0x20|0x10|0x08) >> 3:
+    case (0x80|0x40|0x20|0x10|0x00) >> 3:
+    case (0x80|0x00|0x20|0x10|0x00) >> 3:
+  Q046_long:
+        /* lsquic_Q046_parse_packet_in_long_begin */
+        if (bufsz < 14)
+            return -1;
+        p = buf + 5;
+        dcil = p[0] >> 4;
+        if (dcil)
+            dcil += 3;
+        scil = p[0] & 0xF;
+        if (scil)
+            scil += 3;
+        ++p;
+        if (dcil == GQUIC_CID_LEN && scil == 0)
+        {
+            *cid_len = GQUIC_CID_LEN;
+            return (unsigned) (p - buf);
+        }
+        else
+            return -1;
+    /* 1X00 XGGG: */
+    /*
+    case (0x80|0x40|0x00|0x00|0x08) >> 3:
+    case (0x80|0x00|0x00|0x00|0x08) >> 3:
+    case (0x80|0x40|0x00|0x00|0x00) >> 3:
+    case (0x80|0x00|0x00|0x00|0x00) >> 3:
+    case (0x80|0x40|0x00|0x10|0x08) >> 3:
+    case (0x80|0x00|0x00|0x10|0x08) >> 3:
+    case (0x80|0x40|0x00|0x10|0x00) >> 3:
+    case (0x80|0x00|0x00|0x10|0x00) >> 3:
+    case (0x80|0x40|0x20|0x00|0x08) >> 3:
+    case (0x80|0x00|0x20|0x00|0x08) >> 3:
+    case (0x80|0x40|0x20|0x00|0x00) >> 3:
+    case (0x80|0x00|0x20|0x00|0x00) >> 3:
+    */
+    default:
+        /* parse_ietf_v1_or_Q046plus_long_begin */
+        if (buf[4] == (unsigned) '6')
+            goto Q046_long;
+        /* lsquic_Q050_parse_packet_in_long_begin or
+            lsquic_ietf_v1_parse_packet_in_long_begin */
+        if (bufsz < 14)
+            return -1;
+        dcil = buf[5];
+        if (dcil <= MAX_CID_LEN && 6 + dcil < bufsz)
+        {
+            *cid_len = dcil;
+            return 6;
+        }
+        else
+            return -1;
+    /* 01XX XGGG */
+    case (0x00|0x40|0x00|0x00|0x00) >> 3:
+    case (0x00|0x40|0x00|0x00|0x08) >> 3:
+    case (0x00|0x40|0x00|0x10|0x00) >> 3:
+    case (0x00|0x40|0x00|0x10|0x08) >> 3:
+    case (0x00|0x40|0x20|0x00|0x00) >> 3:
+    case (0x00|0x40|0x20|0x00|0x08) >> 3:
+    case (0x00|0x40|0x20|0x10|0x00) >> 3:
+    case (0x00|0x40|0x20|0x10|0x08) >> 3:
+        /* lsquic_ietf_v1_parse_packet_in_short_begin */
+        if (1 + server_cid_len <= bufsz)
+        {
+            *cid_len = server_cid_len;
+            return 1;
+        }
+        else
+            return -1;
+    /* 00XX 0GGG */
+    case (0x00|0x00|0x00|0x00|0x00) >> 3:
+    case (0x00|0x00|0x00|0x10|0x00) >> 3:
+    case (0x00|0x00|0x20|0x00|0x00) >> 3:
+    case (0x00|0x00|0x20|0x10|0x00) >> 3:
+        /* lsquic_Q046_parse_packet_in_short_begin */
+        if (1 + server_cid_len <= bufsz && (buf[0] & 0x40))
+        {
+            *cid_len = server_cid_len;
+            return 1;
+        }
+        else
+            return -1;
+    /* 00XX 1GGG */
+    case (0x00|0x00|0x00|0x00|0x08) >> 3:
+    case (0x00|0x00|0x00|0x10|0x08) >> 3:
+    case (0x00|0x00|0x20|0x00|0x08) >> 3:
+    case (0x00|0x00|0x20|0x10|0x08) >> 3:
+        /* lsquic_gquic_parse_packet_in_begin */
+        if (1 + GQUIC_CID_LEN <= bufsz
+                        && (buf[0] & PACKET_PUBLIC_FLAGS_8BYTE_CONNECTION_ID))
+        {
+            *cid_len = server_cid_len;
+            return 1;
+        }
+        else
+            return -1;
+    }
+}
+
+
 /* See [draft-ietf-quic-transport-28], Section 12.4 (Table 3) */
 const enum quic_ft_bit lsquic_legal_frames_by_level[N_LSQVER][N_ENC_LEVS] =
 {
-    [LSQVER_ID32] = {
+    [LSQVER_I001] = {
     [ENC_LEV_CLEAR] = QUIC_FTBIT_CRYPTO | QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
                     | QUIC_FTBIT_ACK | QUIC_FTBIT_CONNECTION_CLOSE,
     [ENC_LEV_EARLY] = QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
@@ -230,7 +346,37 @@ const enum quic_ft_bit lsquic_legal_frames_by_level[N_LSQVER][N_ENC_LEVS] =
                     | QUIC_FTBIT_MAX_STREAMS | QUIC_FTBIT_STREAM_BLOCKED
                     | QUIC_FTBIT_STREAMS_BLOCKED
                     | QUIC_FTBIT_NEW_CONNECTION_ID | QUIC_FTBIT_STOP_SENDING
+                    | QUIC_FTBIT_PATH_CHALLENGE
+                    | QUIC_FTBIT_DATAGRAM
+                    | QUIC_FTBIT_RETIRE_CONNECTION_ID,
+    [ENC_LEV_INIT]  = QUIC_FTBIT_CRYPTO | QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
+                    | QUIC_FTBIT_ACK| QUIC_FTBIT_CONNECTION_CLOSE,
+    [ENC_LEV_FORW]  = QUIC_FTBIT_CRYPTO | QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
+                    | QUIC_FTBIT_ACK | QUIC_FTBIT_CONNECTION_CLOSE
+                    | QUIC_FTBIT_STREAM | QUIC_FTBIT_RST_STREAM
+                    | QUIC_FTBIT_BLOCKED
+                    | QUIC_FTBIT_MAX_DATA | QUIC_FTBIT_MAX_STREAM_DATA
+                    | QUIC_FTBIT_MAX_STREAMS | QUIC_FTBIT_STREAM_BLOCKED
+                    | QUIC_FTBIT_STREAMS_BLOCKED
+                    | QUIC_FTBIT_NEW_CONNECTION_ID | QUIC_FTBIT_STOP_SENDING
                     | QUIC_FTBIT_PATH_CHALLENGE | QUIC_FTBIT_PATH_RESPONSE
+                    | QUIC_FTBIT_HANDSHAKE_DONE | QUIC_FTBIT_ACK_FREQUENCY
+                    | QUIC_FTBIT_RETIRE_CONNECTION_ID | QUIC_FTBIT_NEW_TOKEN
+                    | QUIC_FTBIT_TIMESTAMP
+                    | QUIC_FTBIT_DATAGRAM
+                    ,
+    },
+    [LSQVER_ID34] = {
+    [ENC_LEV_CLEAR] = QUIC_FTBIT_CRYPTO | QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
+                    | QUIC_FTBIT_ACK | QUIC_FTBIT_CONNECTION_CLOSE,
+    [ENC_LEV_EARLY] = QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
+                    | QUIC_FTBIT_STREAM | QUIC_FTBIT_RST_STREAM
+                    | QUIC_FTBIT_BLOCKED | QUIC_FTBIT_CONNECTION_CLOSE
+                    | QUIC_FTBIT_MAX_DATA | QUIC_FTBIT_MAX_STREAM_DATA
+                    | QUIC_FTBIT_MAX_STREAMS | QUIC_FTBIT_STREAM_BLOCKED
+                    | QUIC_FTBIT_STREAMS_BLOCKED
+                    | QUIC_FTBIT_NEW_CONNECTION_ID | QUIC_FTBIT_STOP_SENDING
+                    | QUIC_FTBIT_PATH_CHALLENGE
                     | QUIC_FTBIT_DATAGRAM
                     | QUIC_FTBIT_RETIRE_CONNECTION_ID,
     [ENC_LEV_INIT]  = QUIC_FTBIT_CRYPTO | QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
@@ -280,34 +426,6 @@ const enum quic_ft_bit lsquic_legal_frames_by_level[N_LSQVER][N_ENC_LEVS] =
                     | QUIC_FTBIT_DATAGRAM
                     ,
     },
-    [LSQVER_ID28] = {
-    [ENC_LEV_CLEAR] = QUIC_FTBIT_CRYPTO | QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
-                    | QUIC_FTBIT_ACK | QUIC_FTBIT_CONNECTION_CLOSE,
-    [ENC_LEV_EARLY] = QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
-                    | QUIC_FTBIT_STREAM | QUIC_FTBIT_RST_STREAM
-                    | QUIC_FTBIT_BLOCKED | QUIC_FTBIT_CONNECTION_CLOSE
-                    | QUIC_FTBIT_MAX_DATA | QUIC_FTBIT_MAX_STREAM_DATA
-                    | QUIC_FTBIT_MAX_STREAMS | QUIC_FTBIT_STREAM_BLOCKED
-                    | QUIC_FTBIT_STREAMS_BLOCKED
-                    | QUIC_FTBIT_NEW_CONNECTION_ID | QUIC_FTBIT_STOP_SENDING
-                    | QUIC_FTBIT_PATH_CHALLENGE | QUIC_FTBIT_PATH_RESPONSE
-                    | QUIC_FTBIT_RETIRE_CONNECTION_ID,
-    [ENC_LEV_INIT]  = QUIC_FTBIT_CRYPTO | QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
-                    | QUIC_FTBIT_ACK| QUIC_FTBIT_CONNECTION_CLOSE,
-    [ENC_LEV_FORW]  = QUIC_FTBIT_CRYPTO | QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
-                    | QUIC_FTBIT_ACK | QUIC_FTBIT_CONNECTION_CLOSE
-                    | QUIC_FTBIT_STREAM | QUIC_FTBIT_RST_STREAM
-                    | QUIC_FTBIT_BLOCKED
-                    | QUIC_FTBIT_MAX_DATA | QUIC_FTBIT_MAX_STREAM_DATA
-                    | QUIC_FTBIT_MAX_STREAMS | QUIC_FTBIT_STREAM_BLOCKED
-                    | QUIC_FTBIT_STREAMS_BLOCKED
-                    | QUIC_FTBIT_NEW_CONNECTION_ID | QUIC_FTBIT_STOP_SENDING
-                    | QUIC_FTBIT_PATH_CHALLENGE | QUIC_FTBIT_PATH_RESPONSE
-                    | QUIC_FTBIT_HANDSHAKE_DONE | QUIC_FTBIT_ACK_FREQUENCY
-                    | QUIC_FTBIT_RETIRE_CONNECTION_ID | QUIC_FTBIT_NEW_TOKEN
-                    | QUIC_FTBIT_TIMESTAMP
-                    ,
-    },
     [LSQVER_ID27] = {
     [ENC_LEV_CLEAR] = QUIC_FTBIT_CRYPTO | QUIC_FTBIT_PADDING | QUIC_FTBIT_PING
                     | QUIC_FTBIT_ACK | QUIC_FTBIT_CONNECTION_CLOSE,
