@@ -1,4 +1,4 @@
-/* Copyright (c) 2017 - 2019 LiteSpeed Technologies Inc.  See LICENSE. */
+/* Copyright (c) 2017 - 2021 LiteSpeed Technologies Inc.  See LICENSE. */
 /*
  * lsquic_parse_gquic_common.c -- Parsing functions common to GQUIC
  */
@@ -21,10 +21,8 @@
 #include "lsquic_packet_out.h"
 #include "lsquic_packet_gquic.h"
 #include "lsquic_packet_in.h"
-#include "lsquic_packet_out.h"
 #include "lsquic_parse_common.h"
 #include "lsquic_parse.h"
-#include "lsquic_parse_common.h"
 #include "lsquic_version.h"
 #include "lsquic.h"
 
@@ -162,7 +160,7 @@ static const unsigned char simple_prst_payload[] = {
 
 
 typedef char correct_simple_prst_size[(GQUIC_RESET_SZ ==
-                1 + GQUIC_CID_LEN + sizeof(simple_prst_payload)) - 1];
+                1 + GQUIC_CID_LEN + sizeof(simple_prst_payload)) ? 1 : -1 ];
 
 
 ssize_t
@@ -198,7 +196,7 @@ lsquic_generate_gquic_reset (const lsquic_cid_t *cidp,
 }
 
 
-static const enum quic_frame_type byte2frame_type_Q035_thru_Q039[0x100] =
+static const enum quic_frame_type byte2frame_type_Q035_thru_Q046[0x100] =
 {
     [0x00] = QUIC_FRAME_PADDING,
     [0x01] = QUIC_FRAME_RST_STREAM,
@@ -460,14 +458,18 @@ static const enum quic_frame_type byte2frame_type_Q035_thru_Q039[0x100] =
 
 
 enum quic_frame_type
-parse_frame_type_gquic_Q035_thru_Q039 (unsigned char b)
+lsquic_parse_frame_type_gquic_Q035_thru_Q046 (const unsigned char *buf,
+                                                                size_t len)
 {
-    return byte2frame_type_Q035_thru_Q039[b];
+    if (len > 0)
+        return byte2frame_type_Q035_thru_Q046[buf[0]];
+    else
+        return QUIC_FRAME_INVALID;
 }
 
 
 void
-lsquic_turn_on_fin_Q035_thru_Q039 (unsigned char *stream_header)
+lsquic_turn_on_fin_Q035_thru_Q046 (unsigned char *stream_header)
 {
     /* 1fdoooss */
     *stream_header |= 0x40;
@@ -475,7 +477,7 @@ lsquic_turn_on_fin_Q035_thru_Q039 (unsigned char *stream_header)
 
 
 size_t
-calc_stream_frame_header_sz_gquic (lsquic_stream_id_t stream_id,
+lsquic_calc_stream_frame_header_sz_gquic (lsquic_stream_id_t stream_id,
                                     uint64_t offset, unsigned data_sz_IGNORED)
 {
     return
@@ -508,29 +510,12 @@ static const char *const ecn2str[4] =
 };
 
 
-#define ECN_COUNTS  " ECT(0): 01234567879012345678790;" \
-                    " ECT(1): 01234567879012345678790;" \
-                    " CE: 01234567879012345678790"
-#define RANGES_TRUNCATED " ranges truncated! "
-
-char *
-acki2str (const struct ack_info *acki, size_t *sz)
+void
+lsquic_acki2str (const struct ack_info *acki, char *buf, size_t bufsz)
 {
-    size_t off, bufsz, nw;
+    size_t off, nw;
     enum ecn ecn;
     unsigned n;
-    char *buf;
-
-    bufsz = acki->n_ranges * (3 /* [-] */ + 20 /* ~0ULL */ * 2)
-        + (acki->flags & AI_ECN ? sizeof(ECN_COUNTS) : 0)
-        + (acki->flags & AI_TRUNCATED ? sizeof(RANGES_TRUNCATED) : 0);
-    buf = malloc(bufsz);
-    if (!buf)
-    {
-        LSQ_WARN("%s: malloc(%zd) failure: %s", __func__, bufsz,
-                                                        strerror(errno));
-        return NULL;
-    }
 
     off = 0;
     for (n = 0; n < acki->n_ranges; ++n)
@@ -538,18 +523,15 @@ acki2str (const struct ack_info *acki, size_t *sz)
         nw = snprintf(buf + off, bufsz - off, "[%"PRIu64"-%"PRIu64"]",
                 acki->ranges[n].high, acki->ranges[n].low);
         if (nw > bufsz - off)
-            break;
+            return;
         off += nw;
     }
 
     if (acki->flags & AI_TRUNCATED)
     {
-        nw = snprintf(buf + off, bufsz - off, RANGES_TRUNCATED);
+        nw = snprintf(buf + off, bufsz - off, RANGES_TRUNCATED_STR);
         if (nw > bufsz - off)
-        {
-            nw = bufsz - off;
-            goto end;
-        }
+            return;
         off += nw;
     }
 
@@ -560,14 +542,10 @@ acki2str (const struct ack_info *acki, size_t *sz)
             nw = snprintf(buf + off, bufsz - off, " %s: %"PRIu64"%.*s",
                         ecn2str[ecn], acki->ecn_counts[ecn], ecn < 3, ";");
             if (nw > bufsz - off)
-                break;
+                return;
             off += nw;
         }
     }
-
-  end:
-    *sz = off;
-    return buf;
 }
 
 
@@ -596,7 +574,8 @@ lsquic_gquic_packout_size (const struct lsquic_conn *conn,
 
 size_t
 lsquic_gquic_packout_header_size (const struct lsquic_conn *conn,
-                                enum packet_out_flags flags, size_t dcid_len)
+                                enum packet_out_flags flags, size_t dcid_len,
+                                enum header_type unused)
 {
     return lsquic_gquic_po_header_sz(flags);
 }
@@ -654,3 +633,81 @@ lsquic_gquic_calc_packno_bits (lsquic_packno_t packno,
 
     return bits;
 }
+
+
+/* `dst' serves both as source and destination.  `src' is the new frame */
+int
+lsquic_merge_acks (struct ack_info *dst, const struct ack_info *src)
+{
+    const struct lsquic_packno_range *a, *a_end, *b, *b_end, **p;
+    struct lsquic_packno_range *out, *out_end;
+    unsigned i;
+    int ok;
+    struct lsquic_packno_range out_ranges[256];
+
+    if (!(dst->n_ranges && src->n_ranges))
+        return -1;
+
+    a = dst->ranges;
+    a_end = a + dst->n_ranges;
+    b = src->ranges;
+    b_end = b + src->n_ranges;
+    out = out_ranges;
+    out_end = out + sizeof(out_ranges) / sizeof(out_ranges[0]);
+
+    if (a->high >= b->high)
+        *out = *a;
+    else
+        *out = *b;
+
+    while (1)
+    {
+        if (a < a_end && b < b_end)
+        {
+            if (a->high >= b->high)
+                p = &a;
+            else
+                p = &b;
+        }
+        else if (a < a_end)
+            p = &a;
+        else if (b < b_end)
+            p = &b;
+        else
+        {
+            ++out;
+            break;
+        }
+
+        if ((*p)->high + 1 >= out->low)
+            out->low = (*p)->low;
+        else if (out + 1 < out_end)
+            *++out = **p;
+        else
+            return -1;
+        ++*p;
+    }
+
+    if (src->flags & AI_ECN)
+    {
+        /* New ACK frame (src) should not contain ECN counts that are smaller
+         * than previous ACK frame, otherwise we cannot merge.
+         */
+        ok = 1;
+        for (i = 0; i < sizeof(src->ecn_counts)
+                                        / sizeof(src->ecn_counts[0]); ++i)
+            ok &= dst->ecn_counts[i] <= src->ecn_counts[i];
+        if (ok)
+            for (i = 0; i < sizeof(src->ecn_counts)
+                                            / sizeof(src->ecn_counts[0]); ++i)
+                dst->ecn_counts[i] = src->ecn_counts[i];
+        else
+            return -1;
+    }
+    dst->flags |= src->flags;
+    dst->lack_delta = src->lack_delta;
+    dst->n_ranges = out - out_ranges;
+    memcpy(dst->ranges, out_ranges, sizeof(out_ranges[0]) * dst->n_ranges);
+
+    return 0;
+}
