@@ -19,9 +19,9 @@ static const unsigned char version_tags[N_LSQVER][4] =
     [LSQVER_098] = { 'Q', '0', '9', '8', },
 #endif
     [LSQVER_ID27] = { 0xFF, 0, 0, 27, },
-    [LSQVER_ID28] = { 0xFF, 0, 0, 28, },
     [LSQVER_ID29] = { 0xFF, 0, 0, 29, },
-    [LSQVER_ID32] = { 0xFF, 0, 0, 32, },
+    [LSQVER_ID34] = { 0xFF, 0, 0, 34, },
+    [LSQVER_I001] = {    0, 0, 0, 1, },
     [LSQVER_VERNEG] = { 0xFA, 0xFA, 0xFA, 0xFA, },
 };
 
@@ -59,9 +59,9 @@ const char *const lsquic_ver2str[N_LSQVER] = {
     [LSQVER_098] = "Q098",
 #endif
     [LSQVER_ID27] = "FF00001B",
-    [LSQVER_ID28] = "FF00001C",
     [LSQVER_ID29] = "FF00001D",
-    [LSQVER_ID32] = "FF000020",
+    [LSQVER_ID34] = "FF000022",
+    [LSQVER_I001] = "00000001",
     [LSQVER_VERNEG] = "FAFAFAFA",
 };
 
